// Dohvatanje HTML elemenata
var ime_igraca = document.getElementById('ime');
var igrac = document.getElementById('igrac');
var nova_igra = document.getElementById('nova_igra');
var vreme = document.getElementById('vreme');
var tabla = document.getElementById('tabla');
var poeni = document.getElementById('poeni');

// Reagovanje na dogadjanje
ime_igraca.addEventListener('focus', dodajIvice); // Element je u fokusu, poziva se funkcija dodajIvice
ime_igraca.addEventListener('blur', skloniIvice); // Element vise nije u fokusu, poziva se funkcija Skloni ivice
ime_igraca.addEventListener('input', promeniIme); // Sadrzaj tekstualnog polja je promenjen, poziva se funkcija promeniIme
nova_igra.addEventListener('click', novaIgra); // Kliknuto je dugme, poziva se funkcija novaIgra

// Dugme za pocetak igre je onesposobljeno
nova_igra.disabled = true;

// Matrica koja ce predstavljati raspored kartica, elementi matrice su 0 ako je polje prazno,
// brojevi od 1 do 8 ce odgovarati rednim brojevima kartica koje ce se naci na poljima
var matrica;

// Indikator koji oznacava da li je igra u toku
var igra_u_toku = 0;

// Promenljive koje ce cuvati elemente koji predstavljaju prvu i drugu odabranu karticu
var prva_kartica;
var druga_kartica;

// Promenljiva koja cuva broj osvojenih poena
var broj_poena;

// Indikator koji odredjuje da li je dozvoljeno okretanje kartica
var dozvoljen_potez = 0;

// Promenljiva koja cuva broj koliko je polja ostalo neotvorenih
var preostalo_polja;

// Promenljiva koja cuva preostali broj sekundi do kraja igre 
var preostalo_vreme = 60;
vreme.innerHTML = preostalo_vreme;



// Dodavanje crvenih ivica oko polja za unos teksta
function dodajIvice()
{
	ime_igraca.style.border = "2px solid red";
}


// Uklanjanje crvenih ivica oko polja za unos teksta
function skloniIvice()
{
	ime_igraca.style.border = "none";
}


// Promena imena
function promeniIme()
{
	// Vrednost unetog polja upisujemo kao sadrzaj elementa <strong id="igrac"></strong>
	igrac.innerHTML = this.value.trim();
	
	// Funkcija trim odseca beline pre i posle teksta niske, svojstvo length cuva duzinu niske.
	// Ako je duzina imena bez belina veca od 0, osposobljava se dugme za novu igru
	if(ime_igraca.value.trim().length > 0)
		nova_igra.disabled = false;
	else
		nova_igra.disabled = true;
}

function novaIgra()
{
	// Inicijalizacija parametara igre
	preostalo_polja = 16;
	dozvoljen_potez = 1;

	broj_poena = 0;
	poeni.innerHTML = 0;

	nova_igra.disabled = true;
	igra_u_toku = 1;
	preostalo_vreme = 60;

	// Undefined je neodredjena vrednost, sluzi kao indikator da vrednost nije dodeljena
	prva_kartica = undefined;
	druga_kartica = undefined;

	inicijalizujMatricu();	

	nacrtajTablu();

	pokreniTajmer();
}


// Funkcija koja je zaduzena za racunanje i prikaz vremena
function pokreniTajmer()
{
	// Ako je igra zavrsena, tajmer se zaustavlja
	if(igra_u_toku == 0)
		return;

	// Preostalo vreme do kraja igre se upisuje kao sadrzaj elementa <strong id="vreme"></strong> 
	vreme.innerHTML = preostalo_vreme;

	// Ako vreme nije isteklo...
	if(preostalo_vreme != 0)
	{
		preostalo_vreme--; // Smanjuje se broj sekundi do kraja

		// Funkcija tajmer poziva funkciju navedenu kao prvi argument,
		// u ovom slucaju poziva ponovo funkciju zaduzenu za tajmer, nakon broja milisekundi,
		// koji je naveden kao drugi argument. U konkretnom slucaju, pozivamo funkciju koja odbrojava
		// vreme do kraja svakih 1000 ms = 1s.
		setTimeout(pokreniTajmer, 1000); 
	}
	else // Ako vreme jeste isteklo...
	{
		alert("KRAJ IGRE");
		igra_u_toku = 0;
		nova_igra.disabled = false;
	}
}


// Funkcija koja crta tablu sa karticama
function nacrtajTablu()
{
	// Brojaci
	var i, j;

	// Eventualni table se brise
	tabla.innerHTML = "";


	// Tabla se popunjava karticama
	for(i = 0; i < 4; i++)
	{

		for(j = 0; j < 4; j++)
		{

			// Pravljenje novog elementa div sa klasom polje
			polje = document.createElement('div');
			polje.setAttribute('class', 'polje');

			// Pravljenje novog elementa img sa vrednostima atributa:
			// src="images/back.png", data-okrenuta="0" data-broj="Element matrice na poziciji i,j."
			// Korisnicki definisani atributi imenuju se u obliku data-NAZIV.
			// atribut data-okrenuta cuva informaciju o tome da li je kartica okrenuta (1) ili nije (0).
			// atribut data-broj cuva informaciju o broju kartica koja se nalazi na toj poziciji
			kartica = document.createElement('img');
			kartica.setAttribute('src', 'images/back.png');
			kartica.setAttribute('data-okrenuta', 0);
			kartica.setAttribute('data-broj', matrica[i][j]);

			// Zadavanje CSS svojstva transition na 0.3s
			kartica.style.transition = '0.3s';

			// Dodavanje dogadjaja vezanih za kartice
			kartica.addEventListener('click', okreniKarticu); // Klik na karticu
			kartica.addEventListener('mouseover', prikaziSenku); // Prelaz kursora preko kartice
			kartica.addEventListener('mouseleave', skloniSenku); // Izlazak kursora sa kartice


			// Elementu div sa klasom polje dodeljujemo sin cvor sa slikom
			polje.appendChild(kartica);

			// Dodajemo div sa karticom elementu tabla.
			tabla.appendChild(polje);
		}
	}
}

// Funkcija kojom se obradjuje odabir kartice
function okreniKarticu()
{
	// Karticu je moguce okrenuti ako je igra u toku, okretanje dozvoljeno i nije jos okrenuta
	if(igra_u_toku == 1 && dozvoljen_potez == 1 && this.getAttribute('data-okrenuta') == 0)
	{
		// Postavlja se odgovarajuca slika kartice i indikator da je okrenuta
		this.setAttribute('src', 'images/'+this.getAttribute('data-broj')+'.png');
		this.setAttribute('data-okrenuta', 1);

		// Ako jos nije izabrana prva kartica...
		if(prva_kartica == undefined)
		{
			prva_kartica = this; // Postavlja se vrednost na element koji je pozvao funkciju
			// - na karticu za koju je registrovan klik (this)
		}
		else // Ako prva kartica jeste izabrana, ali druga nije
		{
			druga_kartica = this;

			// Ako su kartice iste - vrednosti atributa data-broj su im jednake
			if(prva_kartica.getAttribute('data-broj') == druga_kartica.getAttribute('data-broj'))
			{
				// Azuriranje poena
				broj_poena += 10;
				poeni.innerHTML = broj_poena;

				// Deselektovanje kartica
				prva_kartica = undefined;
				druga_kartica = undefined;

				preostalo_polja -= 2;

				// Ako nema vise neotvorenih polja na tabli igra je zavrsena
				if(preostalo_polja == 0)
				{
					alert("POBEDA!");
					igra_u_toku = 0;
					nova_igra.disabled = false;
				}
			}
			else // U odabranom paru kartice nisu iste
			{
				dozvoljen_potez = 0; // Onesposobljava se klik na ostale kartice
				setTimeout(function(){ // Postavlja se tajmer na 1s nakon cega se
					// neuparene kartice ponovo okrecu
					
					// Okretanje kartica na skrivene
					prva_kartica.setAttribute('data-okrenuta', 0);
					druga_kartica.setAttribute('data-okrenuta', 0);

					// Postavljanje slike poledjine kartica
					prva_kartica.setAttribute('src', 'images/back.png');
					druga_kartica.setAttribute('src', 'images/back.png');

					// Deselektovanje kartica
					prva_kartica = undefined;
					druga_kartica = undefined;
					dozvoljen_potez = 1;

				}, 1000);
			}
		}
	}
}


// Inicijalizacija matrice sa rasporedom kartica
function inicijalizujMatricu()
{
	// Definisanje niza
	matrica = [];
	// moze i sa:
	// matrica = new Array()

	for(i = 0; i < 4; i++)
	{
		matrica[i] = []; // Za i-ti element niza navodimo da je i on novi niz (matrica je niz nizova)

		for(j = 0; j < 4; j++)
		{
			matrica[i][j] = 0; // Inicijalizacija svih polja matrice na 0
		}
	}

	var i1, j1, i2, j2;

	// Smestanje kartica od 1 do 8 na po dva prazna polja na tabli
	for(var k = 1; k <= 8; k++)
	{
		do // Pronalazenje prvog slobodnog polja
		{
			// koordinata i
			i1 = Math.random(); // Generisanje slucajnog broja izmedju 0 i 1
			i1 = Math.trunc(i1 * 100) % 4; // Dobijanje broja u intervalu [0, 3], trunc(...) odseca decimale

			// koordinata j
			j1 = Math.random();
			j1 = Math.trunc(j1 * 100) % 4;

		} while(matrica[i1][j1] != 0) // Sve dok se ne pronadju koordinate polja koje je nije zauzeto

		matrica[i1][j1] = k; // Postavljanje kartice k na polje sa koordinatama (i1, j1)

		do // Pronalazenje drugog slobodnog polja
		{

			i2 = Math.random();
			i2 = Math.trunc(i2 * 100) % 4;

			j2 = Math.random();
			j2 = Math.trunc(j2 * 100) % 4;

		} while(matrica[i2][j2] != 0)

		matrica[i2][j2] = k;
	}
}


// Postavljanje bele senke iza kartice
function prikaziSenku()
{
	this.style.boxShadow = '0px 0px 15px 3px white';
}

// Uklanjanje senke iza kartice
function skloniSenku()
{
	this.style.boxShadow = 'none';
}